// hardtopEA
// version 1d
// 14-JAN-09
// by jingodelcuyo
#property copyright "free"

#include <stdlib.mqh>
#include <stderror.mqh>
#include <WinUser32.mqh>

// Hour1 and Hour2 determine the two daily trading times
extern int Hour1=0;
extern int Hour2=9;

// The remaining external variables are self-explanatory
extern double Lot=0.5;
extern int BuyStop=20;
extern int SellStop=20;
extern int TakeProfit=20;
extern int StopLoss=10;
extern int StopModType=1; // none=0, quantum=1, trail=2
extern int QuantumLevel=12;
extern int QuantumStop=2;
extern int TrailStop=10;
extern bool TradeMonday=true;

// Trade accounting variables
bool won;
bool orderOpened[2];
double old[2][2];
double new[2][2];
int otc[6];
int hr[2];

int init() {
   switch (StopModType) {
      case 0: Alert("Stop Modification Type (StopModType): None"); break;
      case 1: Alert("Stop Modification Type (StopModType): Quantum"); break;
      case 2: Alert("Stop Modification Type (StopModType): Trail"); break;
   }
   // Just in case the values of Hour1 and Hour2 are reversed...
   if (Hour1<=Hour2) {
      hr[0]=Hour1;
      hr[1]=Hour2;
   } else {
      hr[0]=Hour2;
      hr[1]=Hour1;
   }
   if (Hour1==Hour2) hr[1]=hr[0]+1;
   return(0);
}

int deinit() {
   return(0);
}

int start() {
   if (DayOfWeek()==5 && Hour()>=22) deletePendingOrders();
   if (DayOfWeek()==0 || DayOfWeek()==6 || (DayOfWeek()==1 && TradeMonday==false)) return(0);
   orderTracking(); // Basic trade accounting
   events(0); // update trade status; look for trades that have closed TP (won)
   if (won==true) deletePendingOrders(); // If one of the stop orders wins, delete the other one
   if (Hour()==hr[0] && won==false && orderOpened[0]==false) {
      deletePendingOrders();
      openOrders(0); // open the first set of stop orders
   }
   if (Hour()==hr[1] && won==false && orderOpened[1]==false) {
      deletePendingOrders();
      openOrders(1); // open the second set of stop orders
   }
   if (Hour()>hr[1]) { // reset timing/order criteria // MIGHT BE A PROBLEM IF hr[1]=23
      won=false;
      orderOpened[0]=false;
      orderOpened[1]=false;
   }
   switch (StopModType) {
      case 0: break;
      case 1: quantumStop(); break;
      case 2: trailStop(); break;
   }
   return(0);
}

int openOrders(int id) { // open a buy stop and a sell stop
   double OP, SL, TP;
   OP=Ask+BuyStop*Point;
   TP=OP+TakeProfit*Point;
   SL=OP-StopLoss*Point;
   OrderSend(Symbol(),4,Lot,OP,2,SL,TP,"",0); // error handling
   OP=Bid-SellStop*Point;
   TP=OP-TakeProfit*Point;
   SL=OP+StopLoss*Point;
   OrderSend(Symbol(),5,Lot,OP,2,SL,TP,"",0); // error handling
   orderOpened[id]=true;
   return(0);
}

int deletePendingOrders() { // delete pending orders
   for (int i=0; i<OrdersTotal(); i++) {
      OrderSelect(i,SELECT_BY_POS);
      if (OrderType()==4 || OrderType()==5) {
         if (!OrderDelete(OrderTicket())) {} // error handling
      }
   }
}

int orderTracking() { // update order tracking
   int i=0;
   ArrayCopy(old, new);
   ArrayInitialize(new, 0);
   ArrayInitialize(otc, 0);
   for (int j=0; j<OrdersTotal(); j++) {
      if (OrderSelect(j, SELECT_BY_POS) && OrderSymbol()==Symbol()) {
         if (OrderType()<=1) {
            i++;
            new[i][1]=OrderTicket();
         }
         otc[OrderType()]++;
      }
   }
   new[0][0]=i;
}

int events(int rev) {
   bool open;
   for(int i=1;i<=old[0][0];i++) {
      open=false;
      for(int j=1;j<=new[0][0];j++) {
         if (old[i][1]==new[j][1]) {
            open=true;
            break;
         }
      }
      if (open==false) {
         if (OrderSelect(old[i][1],SELECT_BY_TICKET)) {
            double pips=NormalizeDouble(MathAbs((OrderClosePrice()-OrderOpenPrice())/Point),0);
            if (OrderProfit()>0 && pips==TakeProfit) won=true;
         } else {
            // error handling
         }
      }
   }
   return;
}

int trailStop() { // modify order to update stoploss value
   double d, TS;
   for(int j=1;j<=new[0][0];j++) {
      if (OrderSelect(new[j][1],SELECT_BY_TICKET)) {
         switch (OrderType()) {
            case 0: 
               d=NormalizeDouble((Ask-OrderStopLoss())/Point,0);
               if (d>TrailStop) {
                  TS=Ask-TrailStop*Point;
                  if (!OrderModify(OrderTicket(),OrderOpenPrice(),TS,OrderTakeProfit(),0)) {} // error handling
               }
               break;
            case 1: 
               d=NormalizeDouble((OrderStopLoss()-Bid)/Point,0);
               if (d>TrailStop) {
                  TS=Bid+TrailStop*Point;
                  if (!OrderModify(OrderTicket(),OrderOpenPrice(),TS,OrderTakeProfit(),0)) {} // error handling
               }
               break;
         }
      }
   }
}

int quantumStop() { // modify order to update stoploss value
   double d, TS;
   for(int j=1;j<=new[0][0];j++) {
      if (OrderSelect(new[j][1],SELECT_BY_TICKET)) {
         switch (OrderType()) {
            case 0: 
               if (OrderStopLoss()<OrderOpenPrice()) {
                  d=NormalizeDouble((Ask-OrderOpenPrice())/Point,0);
                  if (d>=QuantumLevel) {
                     TS=OrderOpenPrice()+QuantumStop*Point;
                     if (!OrderModify(OrderTicket(),OrderOpenPrice(),TS,OrderTakeProfit(),0)) {} // error handling
                  }
               }
               break;
            case 1: 
               if (OrderStopLoss()>OrderOpenPrice()) {
                  d=NormalizeDouble((OrderOpenPrice()-Bid)/Point,0);
                  if (d>=QuantumLevel) {
                     TS=OrderOpenPrice()-QuantumStop*Point;
                     if (!OrderModify(OrderTicket(),OrderOpenPrice(),TS,OrderTakeProfit(),0)) {} // error handling
                  }
               }
               break;
         }
      }
   }
}